//
//  Copyright (C) by S.Tomabechi 1997,1999
//  All rights reserved.
//
//  Fast Fourier Transformation class  Ver 1.20
//
//  used for multiplication of large integers
//
//  3 primes must be > 2^31
//
#ifndef __FFT_HPP
#define __FFT_HPP
#include <stdlib.h>
#include <iostream.h>

#ifndef __BORLANDC__
#define _N 0             // offset
#define _LOGN 4
#define _ROOTS 8
#define _INVERSE 20
#define _PRIME 32
#define _CRT   44
#define _HP0P1 56
#define _LP0P1 60
#endif

class Mint ;
class FFT
{
  friend class Mint ;
public:
//public:

  unsigned long N ;
  unsigned long logN ;
  unsigned long* roots[3] ;
  unsigned long inverse[3] ;
  unsigned long prime[3] ;         // modulus
  unsigned long crt[3] ;
  unsigned long hp0p1 ;
  unsigned long lp0p1 ;
  static unsigned long*  d[3] ;
  static unsigned long maxN ;
public:
  FFT() ;                // default constructor
  FFT(unsigned long nsize) ;       // N is number of data
  ~FFT() ;               // destructor

        // multiplication of multiple precision integers
        //  ab <- a*b
  void fft_mul(const Mint& a,const Mint& b,Mint& ab) ;

public:
  bool initialize(unsigned long NN) ;
  void fft(unsigned long* data,unsigned long prime,unsigned long* roots) ;
  void ifft(unsigned long* data,unsigned long prime,unsigned long* roots) ;


// for debug
#ifdef FFT_DEBUG
int ncalled ;
#endif
} ;

inline FFT::FFT()
{
  N=0 ;
  roots[0]=roots[1]=roots[2]=NULL ;
//  work=NULL ;
#ifdef FFT_DEBUG
ncalled=0 ;
#endif
//cout << endl << N ;
}


inline FFT::FFT(unsigned long nsize)           // constructor  nsize : number of data
{
  N=0 ;
  roots[0]=roots[1]=roots[2]=NULL ;
//  work=NULL ;
  initialize(nsize) ;
#ifdef FFT_DEBUG
ncalled=0 ;
#endif
}

#endif
