//
//   Rational number class Ver 2.00
//   Copyright (C) by S.Tomabechi 1993,1995,1997
//   All rights reserved.
//
//   1993.October
//   Rational number class.
//   base integer is Mint.
//
//

#if !defined( __RATIONAL_HPP )
#define __RATIONAL_HPP

#include <iostream.h>

#define _ONLY_MINTDECL_
#define _ONLY_FINTDECL_
#include "Mint.hpp"
#include "Fint.hpp"

#include "Error.hpp"
void Swap(Mint& m,Mint& n) ;
unsigned long gcd(const unsigned long m,const unsigned long n) ;
long gcd(const long m,const long n) ;


class Rational
{
  friend class Fint ;
public:

  Rational() ;                   // default 0/1
  Rational(const short n) ;
  Rational(const int n) ;
  Rational(const unsigned long n) ;
  Rational(const long n) ;
  Rational(const Mint& n) ;            // define a/1
  Rational(const Mint& a,const Mint& b) ;
  Rational(const Rational& n) ;

  Rational& operator=(const Rational& a) ;
  Rational& operator=(const Mint& n) ;
  Rational& operator=(const short n) ;
  Rational& operator=(const int n) ;
  Rational& operator=(const long n) ;
  Rational& operator=(const unsigned long n) ;

  Rational& operator+=(const Rational& a) ;
  Rational& operator+=(const Mint& n) ;
  Rational& operator+=(const short n) ;
  Rational& operator+=(const int n) ;
  Rational& operator+=(const long n) ;
  Rational& operator+=(const unsigned long n) ;

  Rational& operator-=(const Rational& a) ;
  Rational& operator-=(const Mint& n) ;
  Rational& operator-=(const short n) ;
  Rational& operator-=(const int n) ;
  Rational& operator-=(const long n) ;
  Rational& operator-=(const unsigned long n) ;

  Rational& operator*=(const Rational& a) ;
  Rational& operator*=(const Mint& n) ;
  Rational& operator*=(const short n) ;
  Rational& operator*=(const int n) ;
  Rational& operator*=(const long n) ;
  Rational& operator*=(const unsigned long n) ;

  Rational& operator/=(const Rational& a) ;
  Rational& operator/=(const Mint& n) ;
  Rational& operator/=(const short n) ;
  Rational& operator/=(const int n) ;
  Rational& operator/=(const long n) ;
  Rational& operator/=(const unsigned long n) ;


            operator Fint() const ;


  bool operator==(const Rational& q) const ;
  bool operator==(const Mint& n) const ;
  bool operator==(const short n) const ;
  bool operator==(const int n) const ;
  bool operator==(const long n) const ;
  bool operator==(const unsigned long n) const ;

  bool operator!=(const Rational& q) const ;
  bool operator!=(const Mint& n) const ;
  bool operator!=(const long n) const ;

  bool operator>(const Rational& q) const ;
  bool operator>(const Mint& n) const ;
  bool operator>(const long n) const ;

  bool operator<(const Rational& q) const ;
  bool operator<(const Mint& n) const ;
  bool operator<(const long n) const ;

  bool operator>=(const Rational& q) const ;
  bool operator>=(const Mint& n) const ;
  bool operator>=(const long n) const ;

  bool operator<=(const Rational& q) const ;
  bool operator<=(const Mint& n) const ;
  bool operator<=(const long n) const ;

  Rational operator-() const ;
  Rational& operator~() ;          // x=-x

  ~Rational() ;                 // do nothing

  const Mint& num() const ;
  const Mint& den() const ;

  Mint& num() ;
  Mint& den() ;

  friend Rational operator+(const Rational& a,const Rational& b) ;
  friend Rational operator+(const Rational& a,const Mint& n) ;
  friend Rational operator+(const Rational& a,const short n) ;
  friend Rational operator+(const Rational& a,const int n) ;
  friend Rational operator+(const Rational& a,const long n) ;
  friend Rational operator+(const Rational& a,const unsigned long n) ;

  friend Rational operator+(const Mint& n,const Rational& q) ;
  friend Rational operator+(const short n,const Rational& q) ;
  friend Rational operator+(const int n,const Rational& q) ;
  friend Rational operator+(const long n,const Rational& q) ;
  friend Rational operator+(const unsigned long a,const Rational& b) ;

  friend Rational operator-(const Rational& a,const Rational& b) ;
  friend Rational operator-(const Rational& a,const Mint& n) ;
  friend Rational operator-(const Rational& a,const short n) ;
  friend Rational operator-(const Rational& a,const int n) ;
  friend Rational operator-(const Rational& a,const long n) ;
  friend Rational operator-(const Rational& a,const unsigned long n) ;

  friend Rational operator-(const Mint& n,const Rational& q) ;
  friend Rational operator-(const long n,const Rational& q) ;


  friend Rational operator*(const Rational& a,const Rational& b) ;
  friend Rational operator*(const Rational& a,const Mint& b) ;
  friend Rational operator*(const Rational& a,const short b) ;
  friend Rational operator*(const Rational& a,const int b) ;
  friend Rational operator*(const Rational& a,const long b) ;
  friend Rational operator*(const Rational& a,const unsigned long b) ;


  friend Rational operator*(const Mint& n,const Rational& q) ;
  friend Rational operator*(const short n,const Rational& q) ;
  friend Rational operator*(const int n,const Rational& q) ;
  friend Rational operator*(const long n,const Rational& q) ;
  friend Rational operator*(const unsigned long n,const Rational& q) ;


  friend Rational operator/(const Rational& a,const Rational& b) ;
  friend Rational operator/(const Rational& a,const Mint& b) ;
  friend Rational operator/(const Rational& a,const short b) ;
  friend Rational operator/(const Rational& a,const int b) ;
  friend Rational operator/(const Rational& a,const long b) ;
  friend Rational operator/(const Rational& a,const unsigned long b) ;

  friend Rational operator/(const long n,const Rational& q) ;
  friend Rational operator/(const int n,const Rational& q) ;
  friend Rational operator/(const Mint& n,const Rational& q) ;
  friend Rational operator/(const long n,const Rational& q) ;
  friend Rational operator/(const int n,const Rational& q) ;

  friend ostream& operator<<(ostream& os,const Rational& r) ;
  friend istream& operator>>(istream& is,Rational& r) ;
  friend void write(ostream& os,const Rational& q) ;
  friend void read(istream& is,Rational& q) ;
  friend void Swap(Rational& n,Rational& m) ;

private:
  Rational& isAdd(const Rational& a,const Rational& b) ;
  Rational& isAdd(const Rational& r,const Mint& m) ;
  Rational& isAdd(const Rational& r,const short n) ;
  Rational& isAdd(const Rational& r,const int n) ;
  Rational& isAdd(const Rational& r,const long n) ;
  Rational& isAddUL(const Rational& r,const unsigned long n) ;

  Rational& isSub(const Rational& a,const Rational& b) ;
  Rational& isSub(const Rational& r,const Mint& m) ;
  Rational& isSub(const Rational& r,const short n) ;
  Rational& isSub(const Rational& r,const int n) ;
  Rational& isSub(const Rational& r,const long n) ;
  Rational& isSubUL(const Rational& r,const unsigned long n) ;

  Rational& isMul(const Rational& a,const Rational& b) ;
  Rational& isMul(const Rational& a,const Mint& n) ;
  Rational& isMul(const Rational& a,const short b) ;
  Rational& isMul(const Rational& a,const int b) ;
  Rational& isMul(const Rational& a,const long n) ;
  Rational& isMulUL(const Rational& a,const unsigned long n) ;

  Rational& isDiv(const Rational& a,const Rational& b) ;
  Rational& isDiv(const Rational& a,const Mint& n) ;
  Rational& isDiv(const Rational& a,const short b) ;
  Rational& isDiv(const Rational& a,const int b) ;
  Rational& isDiv(const Rational& a,const long n) ;
  Rational& isDivUL(const Rational& a,const unsigned long n) ;

  Mint Num ;                     // ʬ
  Mint Den ;                     // ʬ
  static Rational _Rtemp ;       // ݥѤѿ

  Rational& normalize(void) ;
  int compare(const Rational& q) const ;
  int compare(const Mint& n) const ;
  int compare(const long n) const ;

} ;

inline Rational& Rational::operator~()         // x=-x
{
    ~Den ;
    return *this ;
}

inline Rational::Rational()
{
  Num=0 ;
  Den=1 ;
}

inline Rational::Rational(const short n)
{
  Num=n ;
  Den=1 ;
}

inline Rational::Rational(const int n)
{
  Num=n ;
  Den=1 ;
}

inline Rational::Rational(const long n)
{
  Num=n ;
  Den=1 ;
}

inline Rational::Rational(const unsigned long n)
{
  Num=n ;
  Den=1 ;
}

inline Rational::Rational(const Mint& a)
{
  Num=a ;
  Den=1 ;
}

inline Rational::Rational(const Mint& a,const Mint& b)   // define a/b
{
  Num=a ;
  Den=b ;
  normalize() ;
}

inline Rational::Rational(const Rational& q)      // copy constructor
{
  Num=q.Num ;
  Den=q.Den ;
}

inline Rational::~Rational(){}

inline Rational& Rational::operator=(const Rational& q)
{
  Num=q.Num ;
  Den=q.Den ;
  return *this ;
}

inline Rational& Rational::operator=(const Mint& n)
{
  Num=n ;
  Den=1 ;
  return *this ;
}

inline Rational& Rational::operator=(const short n)
{
  return operator=(long(n)) ;
}

inline Rational& Rational::operator=(const int n)
{
  return operator=(long(n)) ;
}

inline Rational& Rational::operator=(const long n)
{
  Num=n ;
  Den=1 ;
  return *this ;
}

inline Rational& Rational::operator=(const unsigned long n)
{
  Num=n ;
  Den=1 ;
  return *this ;
}

inline Rational& Rational::isAdd(const Rational& a,const Rational& b)
{
  Mint::_uuMtemp.isMul(a.Den,b.Den) ;
//  Mint::_Mtemp.isMul(a.Num,b.Den) ;
  Num.isAdd(Mint::_Mtemp.isMul(a.Num,b.Den),Mint::_vvMtemp.isMul(a.Den,b.Num)) ;
  Den=Mint::_uuMtemp ;

//Mint::_uuMtemp=a.Den*b.Den ;
//Num=a.Num*b.Den+a.Den*b.Num ;
//Den=Mint::_uuMtemp ;
//  (Num += Mint::_Mtemp.isMul(a.Num,b.Den))
//     += Mint::_vvMtemp.isMul(a.Den,b.Num) ;
//  Num=a.Num*b.Den+a.Den*b.Num ;
  return normalize() ;
}

inline Rational& Rational::isAdd(const Rational& r,const Mint& m)
{
  Num.isAdd(r.Num,Mint::_Mtemp.isMul(r.Den,m)) ;
  return normalize() ;
}

inline Rational& Rational::isAdd(const Rational& r,const short n)
{
  return isAdd(r,long(n)) ;
}

inline Rational& Rational::isAdd(const Rational& r,const int n)
{
  return isAdd(r,long(n)) ;
}

inline Rational& Rational::isAdd(const Rational& r,const long n)
{
  Num.isAdd(r.Num,Mint::_Mtemp.isMul(r.Den,n)) ;
  return normalize() ;
}

inline Rational& Rational::isAddUL(const Rational& r,const unsigned long n)
{
  Num.isAdd(r.Num,Mint::_Mtemp.isMulUL(r.Den,n)) ;
  return normalize() ;
}

inline Rational& Rational::isSub(const Rational& a,const Rational& b)
{
  Mint::_uuMtemp.isMul(a.Den,b.Den) ;
 // (Num += Mint::_vvMtemp.isMul(a.Num,b.Den))
 //    -= Mint::_vvMtemp.isMul(a.Den,b.Num) ;
  Num.isSub(Mint::_Mtemp.isMul(a.Num,b.Den),Mint::_vvMtemp.isMul(a.Den,b.Num)) ;
  Den=Mint::_uuMtemp ;
  return normalize() ;
}


inline Rational& Rational::isSub(const Rational& r,const Mint& m)
{
  Num.isSub(r.Num,Mint::_uuMtemp.isMul(m,r.Den)) ;
  return normalize() ;
}

inline Rational& Rational::isSub(const Rational& r,const short n)
{
  return isAdd(r,long(n)) ;
}

inline Rational& Rational::isSub(const Rational& r,const int n)
{
  return isAdd(r,long(n)) ;
}

inline Rational& Rational::isSub(const Rational& r,const long n)
{
  Num.isSub(r.Num,Mint::_uuMtemp.isMul(r.Den,n)) ;
  return normalize() ;
}

inline Rational& Rational::isSubUL(const Rational& r,const unsigned long n)
{
  Num.isSub(r.Num,Mint::_Mtemp.isMulUL(r.Den,n)) ;
  return normalize() ;
}

inline Rational& Rational::isMul(const Rational& a,const Rational& b)
{
  Num.isMul(a.Num,b.Num) ;
  Den.isMul(a.Den,b.Den) ;
  return normalize() ;
}

inline Rational& Rational::isMul(const Rational& a,const Mint& n)
{
  Mint::_Mtemp=gcd(a.Den,n) ;
  Num *= Mint::_uuMtemp.isDiv(n,Mint::_Mtemp) ;
  Den /= Mint::_Mtemp ;
  return *this ;
}

inline Rational& Rational::isMul(const Rational& a,const short n)
{
  return isMul(a,long(n)) ;
}

inline Rational& Rational::isMul(const Rational& a,const int n)
{
  return isMul(a,long(n)) ;
}

inline Rational& Rational::isMul(const Rational& a,const long n)
{
  long d=gcd(a.Den%n,n) ;
  Num.isMul(a.Num,n/d) ;
  Den.isDiv(a.Den,d) ;
//  Num*=n/d ;
//  Den/=d ;
  return *this ;
}


inline Rational& Rational::isMulUL(const Rational& a,const unsigned long n)
{
  unsigned long d=gcd((unsigned long)(a.Den%n),n) ;
  Num.isMulUL(a.Num,n/d) ;
  Den.isDivUL(a.Den,d) ;
//  Num *= n/d ;
//  Den /= d ;
  return *this ;
}


inline Rational& Rational::isDiv(const Rational& a,const Rational& b)
{
  if( b.Num.isZero() ) Error(zero divide) ;
  Num.isMul(a.Num,b.Den) ;
  Den.isMul(a.Den,b.Num) ;
//  Num *= a.Den ;
//  Den *= a.Num ;

  return normalize() ;
}

inline Rational& Rational::isDiv(const Rational& a,const Mint& b)
{
  Mint::_Mtemp=gcd(a.Num,b) ;
//  Den *= n/Mint::_Mtemp ;
  Den.isDiv(a.Den,Mint::_uuMtemp.isDiv(b,Mint::_Mtemp)) ;
  Num.isDiv(a.Num,Mint::_Mtemp) ;
  return *this ;
}

inline Rational& Rational::isDiv(const Rational& a,const short n)
{
  return isDiv(a,long(n)) ;
}

inline Rational& Rational::isDiv(const Rational& a,const int n)
{
  return isDiv(a,long(n)) ;
}

inline Rational& Rational::isDiv(const Rational& a,const long n)
{
  long d=gcd(n,a.Num%n) ;
  Den.isMul(a.Den,n/d) ;
  Num.isDiv(a.Num,d) ;
//  Den *= n/d ;
//  Num /= d ;
  return *this ;
}

inline Rational& Rational::isDivUL(const Rational& a,const unsigned long n)
{
  unsigned long d=gcd(n,(unsigned long)(a.Num%n)) ;
  Den.isMulUL(a.Den,n/d) ;
  Num.isDivUL(a.Num,d) ;
//  Den *= n/d ;
//  Num /= d ;
  return *this ;
}

inline Rational& Rational::operator+=(const Rational& a)
{
  return isAdd(*this,a) ;
}

inline Rational& Rational::operator+=(const Mint& n)
{
  return isAdd(*this,n) ;
//  Num += Mint::_Mtemp.isMul(Den,n) ;
//  return normalize() ;
}

inline Rational& Rational::operator+=(const short n)
{
  return isAdd(*this,long(n)) ;
//  Num += Mint::_Mtemp.isMulUL(Den,n) ;
//  return normalize() ;
}

inline Rational& Rational::operator+=(const int n)
{
  return isAdd(*this,long(n)) ;
//  Num += Mint::_Mtemp.isMulUL(Den,n) ;
//  return normalize() ;
}

inline Rational& Rational::operator+=(const long n)
{
  return isAdd(*this,n) ;
//  Num += Mint::_Mtemp.isMulUL(Den,n) ;
//  return normalize() ;
}

inline Rational& Rational::operator+=(const unsigned long n)
{
  return isAddUL(*this,n) ;
//  Num += Mint::_Mtemp.isMulUL(Den,n) ;
//  return normalize() ;
}


/**
inline Rational Rational::operator+(const Rational& a)
{
  return _Rtemp.isAdd(*this,a) ;
}
**/

inline Rational operator+(const Rational& a,const Rational& b)
{
  return Rational::_Rtemp.isAdd(a,b) ;
}

/***
inline Rational Rational::operator+(const Mint& n)
{
  return _Rtemp.isAdd(*this,n) ;
}
***/

inline Rational operator+(const Rational& a,const Mint& n)
{
  return Rational::_Rtemp.isAdd(a,n) ;
}

inline Rational operator+(const Rational& a,const short n)
{
  return Rational::_Rtemp.isAdd(a,n) ;
}

inline Rational operator+(const Rational& a,const int n)
{
  return Rational::_Rtemp.isAdd(a,n) ;
}

inline Rational operator+(const Rational& a,const long n)
{
  return Rational::_Rtemp.isAdd(a,n) ;
}

inline Rational operator+(const Rational& a,const unsigned long n)
{
  return Rational::_Rtemp.isAddUL(a,n) ;
}



inline Rational operator+(const Mint& n,const Rational& q)
{
  return Rational::_Rtemp.isAdd(q,n) ;
}


/**
inline Rational Rational::operator+(const unsigned long n)
{
  return _Rtemp.isAdd(*this,n) ;
}


inline Rational Rational::operator+(const long n)
{
  return _Rtemp.isAdd(*this,n) ;
}
**/



inline Rational operator+(const short n,const Rational& q)
{
  return Rational::_Rtemp.isAdd(q,long(n)) ;
//  return q.operator+(long(n)) ;
}

inline Rational operator+(const int n,const Rational& q)
{
  return Rational::_Rtemp.isAdd(q,long(n)) ;
//  return q.operator+(long(n)) ;
}

inline Rational operator+(const long n,const Rational& q)
{
  return Rational::_Rtemp.isAdd(q,n) ;
// return q.operator+(n) ;
}

inline Rational operator+(const unsigned long n,const Rational& q)
{
  return Rational::_Rtemp.isAddUL(q,n) ;
// return q.operator+(n) ;
}

inline Rational& Rational::operator-=(const Rational& a)
{
  return isSub(*this,a) ;
}

inline Rational& Rational::operator-=(const Mint& n)
{
  return isSub(*this,n) ;
}

inline Rational& Rational::operator-=(const short n)
{
  return isSub(*this,n) ;
}

inline Rational& Rational::operator-=(const int n)
{
  return isSub(*this,n) ;
}

inline Rational& Rational::operator-=(const long n)
{
  return isSub(*this,n) ;
}

inline Rational& Rational::operator-=(const unsigned long n)
{
  return isSubUL(*this,n) ;
}


inline Rational operator-(const Rational& a,const Rational& b)
{
  return Rational::_Rtemp.isSub(a,b) ;
}

inline Rational operator-(const Rational& a,const Mint& b)
{
  return Rational::_Rtemp.isSub(a,b) ;
}

inline Rational operator-(const Rational& a,const long b)
{
  return Rational::_Rtemp.isSub(a,b) ;
}

inline Rational operator-(const Mint& n,const Rational& q)
{
  Rational::_Rtemp.isSub(q,n) ;
  ~Rational::_Rtemp.Num ;
  return Rational::_Rtemp ;
}

inline Rational operator-(const long n,const Rational& q)
{
  Rational::_Rtemp.isSub(q,n) ;
  ~Rational::_Rtemp.Num ;     // Num <- (-Num)
  return Rational::_Rtemp ;
}

inline Rational& Rational::operator*=(const Rational& a)
{
  return isMul(*this,a) ;
}


inline Rational& Rational::operator*=(const Mint& n)
{
  return isMul(*this,n) ;
}

inline Rational& Rational::operator*=(const short n)
{
  return isMul(*this,n) ;
}

inline Rational& Rational::operator*=(const int n)
{
  return isMul(*this,n) ;
}


inline Rational& Rational::operator*=(const long n)
{
  return isMul(*this,n) ;
}


inline Rational& Rational::operator*=(const unsigned long n)
{
  return isMulUL(*this,n) ;
}

inline Rational operator*(const Rational& a,const Rational& b)
{
  return Rational::_Rtemp.isMul(a,b) ;
}


inline Rational operator*(const Rational& a,const Mint& b)
{
  return Rational::_Rtemp.isMul(a,b) ;
}


inline Rational operator*(const Rational& a,const short b)
{
  return Rational::_Rtemp.isMul(a,b) ;
}

inline Rational operator*(const Rational& a,const int b)
{
  return Rational::_Rtemp.isMul(a,b) ;
}

inline Rational operator*(const Rational& a,const long b)
{
  return Rational::_Rtemp.isMul(a,b) ;
}

inline Rational operator*(const Rational& a,const unsigned long b)
{
  return Rational::_Rtemp.isMulUL(a,b) ;
}


inline Rational operator*(const Mint& n,const Rational& q)
{
  return Rational::_Rtemp.isMul(q,n) ;
}

inline Rational operator*(const short n,const Rational& q)
{
  return Rational::_Rtemp.isMul(q,long(n)) ;
}

inline Rational operator*(const int n,const Rational& q)
{
  return Rational::_Rtemp.isMul(q,long(n)) ;
}

inline Rational operator*(const long n,const Rational& q)
{
  return Rational::_Rtemp.isMul(q,n) ;
}

inline Rational operator*(const unsigned long a,const Rational& b)
{
  return Rational::_Rtemp.isMulUL(b,a) ;
}


inline Rational& Rational::operator/=(const Rational& a)
{
  if( a.Num==0 ) Error(zero divide) ;
  Num *= a.Den ;
  Den *= a.Num ;
  return normalize() ;
}

inline Rational& Rational::operator/=(const Mint& n)
{
  Mint::_Mtemp=gcd(n,Num) ;
  Den *= Mint::_uuMtemp.isDiv(n,Mint::_Mtemp) ;
  Num /= Mint::_Mtemp ;
  return *this ;
}

inline Rational& Rational::operator/=(const short n)
{
  return isDiv(*this,n) ;
}

inline Rational& Rational::operator/=(const int n)
{
  return isDiv(*this,n) ;
}

inline Rational& Rational::operator/=(const long n)
{
  return isDiv(*this,n) ;
}

inline Rational& Rational::operator/=(const unsigned long n)
{
  return isDivUL(*this,n) ;
}


inline Rational operator/(const Rational& a,const Rational& b)
{
  return Rational::_Rtemp.isDiv(a,b) ;
}

inline Rational operator/(const Rational& a,const Mint& b)
{
  return Rational::_Rtemp.isDiv(a,b) ;
}

inline Rational operator/(const Rational& a,const short b)
{
  return Rational::_Rtemp.isDiv(a,b) ;
}

inline Rational operator/(const Rational& a,const int b)
{
  return Rational::_Rtemp.isDiv(a,b) ;
}

inline Rational operator/(const Rational& a,const long n)
{
  return Rational::_Rtemp.isDiv(a,n) ;
//  return ( _Rtemp=*this ) /= n ;
}

inline Rational operator/(const Rational& a,const unsigned long n)
{
  return Rational::_Rtemp.isDivUL(a,n) ;
//  return ( _Rtemp=*this ) /= n ;
}



// zero divide åʤФʤʤά
//
inline Rational operator/(const Mint& n,const Rational& q)
{
  Rational::_Rtemp.Num.isMul(n,q.Den) ;
  Rational::_Rtemp.Den=q.Num ;
  return Rational::_Rtemp.normalize() ;
}

inline Rational operator/(const long n,const Rational& q)
{
  Rational::_Rtemp.Num.isMul(q.Den,n) ;
  Rational::_Rtemp.Den=q.Num ;
  return Rational::_Rtemp.normalize() ;
}

inline Rational operator/(const int n,const Rational& q)
{
  return operator/(long(n),q) ;
}

// Fint <- Rational    ͭ¿Ѵ
inline Rational::operator Fint() const
{
  return Num|Den ;
}


inline int Rational::compare(const Rational& q) const
{
  return (Mint::_uuMtemp.isMul(Num,q.Den)).compare(Mint::_vvMtemp.isMul(q.Num,Den)) ;
}

inline int Rational::compare(const Mint& n) const
{
  return Num.compare(Mint::_Mtemp.isMul(n,Den)) ;
}

inline int Rational::compare(const long n) const
{
  return Num.compare(Mint::_Mtemp.isMul(Den,n)) ;
}

inline bool Rational::operator==(const Rational& q) const
{
  return compare(q)==isEqual ;
}

inline bool Rational::operator==(const Mint& n) const
{
  return Den.nDigit==1 && *Den.digit==1 && Num==n  ;
//  return compare(n)==isEqual ;
}

inline bool Rational::operator==(const short n) const
{
  return operator==(long(n)) ;
//  return Den.nDigit==1 && *Den.digit==1 && Num==n  ;
}

inline bool Rational::operator==(const int n) const
{
  return operator==(long(n)) ;
//  return Den.nDigit==1 && *Den.digit==1 && Num==n  ;
}

inline bool Rational::operator==(const long n) const
{
  return Den.nDigit==1 && *Den.digit==1 && Num==n  ;
}

inline bool Rational::operator==(const unsigned long n) const
{
  return Den.nDigit==1 && *Den.digit==1 && Num==n  ;
}

inline bool Rational::operator!=(const Rational& q) const
{
  return !(compare(q)==isEqual) ;
}

inline bool Rational::operator!=(const Mint& n) const
{
  return !(compare(n)==isEqual) ;
}
inline bool Rational::operator!=(const long n) const
{
  return !(compare(n)==isEqual ) ;
}

inline bool Rational::operator>(const Rational& q) const
{
  return compare(q)==isGreater ;
}

inline bool Rational::operator>(const Mint& n) const
{
  return compare(n)==isGreater ;
}
inline bool Rational::operator>(const long n) const
{
  return compare(n)==isGreater ;
}

inline bool Rational::operator<(const Rational& q) const
{
  return compare(q)==isLess ;
}

inline bool Rational::operator<(const Mint& n) const
{
  return compare(n)==isLess ;
}

inline bool Rational::operator<(const long n) const
{
  return compare(n)==isLess ;
}

inline bool Rational::operator>=(const Rational& q) const
{
  return !(compare(q)==isLess) ;
}

inline bool Rational::operator>=(const Mint& n) const
{
  return !(compare(n)==isLess) ;
}

inline bool Rational::operator<=(const Rational& q) const
{
  return !(compare(q)==isGreater) ;
}

inline bool Rational::operator<=(const Mint& n) const
{
  return !(compare(n)==isGreater) ;
}
inline bool Rational::operator<=(const long n) const
{
  return !(compare(n)==isGreater) ;
}

inline Rational Rational::operator-() const
{
  _Rtemp=*this ;
  ~(_Rtemp.Num) ;
  return _Rtemp ;
}

inline const Mint& Rational::num() const
{
  return Num ;
}

inline const Mint& Rational::den() const
{
  return Den ;
}


inline Mint& Rational::num()
{
  return Num ;
}

inline  Mint& Rational::den()
{
  return Den ;
}

inline ostream& operator<<(ostream& os,const Rational& a)
{
  os << a.Num ;
  if( a.Den!=1 ) os << '/' << a.Den ;
  return os ;
}

inline istream& operator>>(istream& is,Rational& a)
{
  char c ;
  is >> a.Num >> c >> a.Den  ;
  return is ;
}

inline void write(ostream& os,const Rational& q)
{
  write(os,q.Num) ;
  write(os,q.Den) ;
}

inline void read(istream& is,Rational& q)
{
  read(is,q.Num) ;
  read(is,q.Den) ;
}

inline void Swap(Rational& n,Rational& m)
{
  Swap(n.Num,m.Num) ;
  Swap(n.Den,m.Den) ;
}

#endif
