//
//   Copyright (C) by S.Tomabechi 1993,1995
//   All rights reserved.
//
//   1993.October
//   RatLong number class.
//   base integer is Mint.
//
//  ̃NX́CقƂǃeXgsĂȂD
//

#if !defined( __RATLONG_HPP )
#define __RATLONG_HPP


#include <iostream.h>

#include "IntFuncs.hpp"

#include "Error.hpp"

#define isLess -1
#define isEqual 0
#define isGreater 1

int compare(long a,long b) ;

class RatLong
{
public:

  RatLong() ;                   // default 1/1
  RatLong(long n) ;
  RatLong(long n,long d) ;
  RatLong(RatLong& q) ;

  RatLong& operator=(RatLong& a) ;
  RatLong& operator=(long a) ;

  RatLong& operator+=(RatLong& a) ;
  RatLong& operator+=(long n) ;

  RatLong  operator+(RatLong& a) ;
  RatLong  operator+(long n) ;

  RatLong& operator-=(RatLong& a) ;
  RatLong& operator-=(long n) ;

  RatLong  operator-(RatLong& a) ;
  RatLong  operator-(long n) ;

  RatLong& operator*=(RatLong& a) ;
  RatLong& operator*=(long n) ;

  RatLong  operator*(RatLong& a) ;
  RatLong  operator*(long n) ;

  RatLong& operator/=(RatLong& a) ;
  RatLong& operator/=(long n) ;

  RatLong  operator/(RatLong& a) ;
  RatLong  operator/(long n) ;

            operator double() ;


  int operator==(RatLong& q) ;
  int operator==(long n) ;
  int operator==(int c) { return *this==long(c) ; }

  int operator!=(RatLong& q) ;
  int operator!=(long n) ;

  int operator>(RatLong& q) ;
  int operator>(long n) ;

  int operator<(RatLong& q) ;
  int operator<(long n) ;

  int operator>=(RatLong& q) ;
  int operator>=(long n) ;

  int operator<=(RatLong& q) ;
  int operator<=(long n) ;

  RatLong operator-() ;

  ~RatLong() ;                 // do nothing

   long& num() ;
   long& den() ;

  friend RatLong operator+(long n,RatLong& q) ;
  friend RatLong operator-(long n,RatLong& q) ;
  friend RatLong operator*(long n,RatLong& q) ;
  friend RatLong operator/(long n,RatLong& q) ;

  friend ostream& operator<<(ostream& os,RatLong& v) ;
  friend istream& operator>>(istream& is,RatLong& v) ;
  friend void write(ostream& os,RatLong& q) ;
  friend void read(istream& is,RatLong& q) ;

private:

  long Num ;                     // q
  long Den ;                     // 
  static RatLong _Rtemp ;

  RatLong& normalize(void) ;
  int compare(RatLong& q) ;
  int compare(long n) ;

} ;

inline RatLong::RatLong()
{
  Num=1 ;
  Den=1 ;
}

inline RatLong::RatLong(long n)
{
  Num=n ;
  Den=1 ;
}

inline RatLong::RatLong(long n,long d)
{
  Num=n ;
  Den=d ;
  normalize() ;
}


inline RatLong::RatLong(RatLong& q)      // copy constructor
{
  Num=q.Num ;
  Den=q.Den ;
}

inline RatLong::~RatLong(){}

inline RatLong& RatLong::operator=(RatLong& q)
{
//  if( this==&q ) return *this ;
  Num=q.Num ;
  Den=q.Den ;
  return *this ;
}

inline RatLong& RatLong::operator=(long q)
{
  Num=q ;
  Den=1 ;
  return *this ;
}

inline RatLong& RatLong::operator+=(RatLong& a)
{
  Num=Num*a.Den+Den*a.Num ;
  Den *= a.Den ;
  return normalize() ;
}

inline RatLong& RatLong::operator+=(long n)
{
  Num += Den*n ;
  return normalize() ;
}


inline RatLong RatLong::operator+(long n)
{
  _Rtemp.Num=Num+n*Den ;
  _Rtemp.Den=Den ;
  return _Rtemp.normalize() ;
}

inline RatLong operator+(long n,RatLong& q)
{
  return q+n ;
}

inline RatLong& RatLong::operator-=(RatLong& a)
{
  Num=Num*a.Den-Den*a.Num ;
  Den *= a.Den ;
  return normalize() ;
}


inline RatLong& RatLong::operator-=(long n)
{
  Num -= n*Den ;
  return normalize() ;
}

inline RatLong RatLong::operator-(RatLong& a)
{
  _Rtemp.Den=Den*a.Den ;
  _Rtemp.Num=Num*a.Den-Den*a.Num ;
  return _Rtemp.normalize() ;
}


inline RatLong operator-(long n,RatLong& q)
{
  RatLong::_Rtemp.Num=n*q.Den-q.Num ;
  RatLong::_Rtemp.Den=q.Den ;
  return RatLong::_Rtemp.normalize() ;
}

inline RatLong& RatLong::operator*=(RatLong& a)
{
  Num *= a.Num ;
  Den *= a.Den ;
  return normalize() ;
}

inline RatLong& RatLong::operator*=(long n)
{
  long d=gcd(Den%n,n) ;
  Num *= n/d ;
  Den /= d ;
  return *this ;
}


inline RatLong RatLong::operator*(RatLong& a)
{
  _Rtemp.Den=Den*a.Den ;
  _Rtemp.Num=Num*a.Num ;
  return _Rtemp.normalize() ;
}

inline RatLong RatLong::operator*(long n)
{
  _Rtemp.Num=n*Num ;
  _Rtemp.Den=Den ;

  return _Rtemp.normalize() ;
}


inline RatLong operator*(long n,RatLong& q)
{
  return q*n ;
}

inline RatLong& RatLong::operator/=(RatLong& a)
{
  if( a.Num==0 ) Error(zero divide) ;
  Num *= a.Den ;
  Den *= a.Num ;
  return normalize() ;
}

inline RatLong& RatLong::operator/=(long n)
{
  long d=gcd(n,Num%n) ;
  Den *= n/d ;
  Num /= d ;
  return *this ;
}

inline RatLong RatLong::operator/(RatLong& a)
{
  return (_Rtemp=*this) /= a ;
}

inline RatLong RatLong::operator/(long n)
{
  return ( _Rtemp=*this ) /= n ;
}


inline RatLong operator/(long n,RatLong& q)
{
  RatLong::_Rtemp.Num=n*q.Den ;
  RatLong::_Rtemp.Den=q.Num ;
  return RatLong::_Rtemp.normalize() ;
}

// Fint <- RatLong    Lɕϊ
inline RatLong::operator double()
{
  return Num|Den ;
}


inline int RatLong::compare(RatLong& q)
{
  return ::compare(Num*q.Den,q.Num*Den) ;
}

inline int RatLong::compare(long n)
{
  return ::compare(Num,n*Den) ;
}

inline int RatLong::operator==(RatLong& q)
{
  return compare(q)==isEqual ;
}

inline int RatLong::operator==(long n)
{
  return compare(n)==isEqual ;
}

inline int RatLong::operator!=(RatLong& q)
{
  return !(compare(q)==isEqual) ;
}

inline int RatLong::operator!=(long n)
{
  return !(compare(n)==isEqual ) ;
}

inline int RatLong::operator>(RatLong& q)
{
  return compare(q)==isGreater ;
}

inline int RatLong::operator>(long n)
{
  return compare(n)==isGreater ;
}

inline int RatLong::operator<(RatLong& q)
{
  return compare(q)==isLess ;
}


inline int RatLong::operator<(long n)
{
  return compare(n)==isLess ;
}

inline int RatLong::operator>=(RatLong& q)
{
  return !(compare(q)==isLess) ;
}


inline int RatLong::operator<=(RatLong& q)
{
  return !(compare(q)==isGreater) ;
}

inline int RatLong::operator<=(long n)
{
  return !(compare(n)==isGreater) ;
}

inline RatLong RatLong::operator-()
{
  _Rtemp.Den=Den ;
  _Rtemp.Num=-Num ;
//  ~(_Rtemp.Num) ;
  return _Rtemp ;
}

inline long& RatLong::num()
{
  return Num ;
}

inline long& RatLong::den()
{
  return Den ;
}

/*************
inline ostream& operator<<(ostream& os,RatLong& a)
{
  os << a.Num << '/' << a.Den ;
  return os ;
}
*************/

inline istream& operator>>(istream& is,RatLong& a)
{
  char c ;
  is >> a.Num >> c >> a.Den  ;
  return is ;
}
/*****
inline void write(ostream& os,RatLong& q)
{
  write(os,q.Num) ;
  write(os,q.Den) ;
}

inline void read(istream& is,RatLong& q)
{
  read(is,q.Num) ;
  read(is,q.Den) ;
}
*********/

#endif
