//
//  Copyright by S.Tomabechi 1993,1994,1997
//  All rights reserved.
//
//  Computes Kronecker-Jacobi symbol
//
//
//  return value (a/b) 1 : a is quadratic residue mod b
//                    -1 : a is not quadratic residue
//                     0 : gcd(a,b)!=1
//
#ifdef __BORLANDC__
#pragma inline
#endif

#include "Mint.hpp"

extern const long _Kr_tab[] ;

#ifdef __BORLANDC__
long Kronecker(const Mint& a,const Mint& b)
{
  long k,v ;

  Mint::_uuMtemp=a ;
  Mint::_vvMtemp=b ;

#ifdef _MEMMAN_DEBUG
Mint::_MemMan.check(__FILE__,__LINE__) ;
#endif

  if( Mint::_vvMtemp.isZero()  ) {                     // b is zero ?
    if( ((unsigned long)Mint::_uuMtemp)==1 && Mint::_uuMtemp.nDigit==1 )
      return 1 ;
    else return 0 ;
  }
Remove_2_from_b:
  if( Mint::_uuMtemp.isEven() && Mint::_vvMtemp.isEven() ) return 0 ;

  v=Val2(Mint::_vvMtemp) ;
  if( v>0 ) Mint::_vvMtemp>>=v ;
  if( (v&1)==0 ) k=1 ;
  else k=_Kr_tab[long(Mint::_uuMtemp)&7] ;

  if( Mint::_vvMtemp.isNegative() ) {                 // b<0 ?
    Mint::_vvMtemp.nDim&=_POSITIVE_ ;       // absolute value
    if( Mint::_uuMtemp.isNegative()<0 ) k=-k ;
  }

  for(;;) {
        // a=0 ?
    if( Mint::_uuMtemp.isZero() ) {
      if( *Mint::_vvMtemp.digit==1 && Mint::_vvMtemp.nDigit==1) return k ;
      else return 0 ;
    }
    v=Val2(Mint::_uuMtemp) ;
    if( v>0 ) {
      Mint::_uuMtemp>>=v ;
      if( v&1 ) k*=_Kr_tab[((unsigned long)Mint::_vvMtemp)&7] ;
    }
     // use reciprocity law (p/q)(q/p)=(-1)^((p-1)(q-1)/4)
    if( long(Mint::_uuMtemp) & ((unsigned long)Mint::_vvMtemp) & 2 )
       k=-k ;
    if( Mint::_uuMtemp.isNegative() )
       Mint::_uuMtemp.nDim&=_POSITIVE_ ;    // r<- |a|

                // calculate (b/a) instead of (a/b)
                // (b/a)=(b%a/a)
    __asm {                                               // b <- r
      mov   eax,dword ptr [@Mint@_uuMtemp]          // a <- b
      xchg  eax,dword ptr [@Mint@_vvMtemp]
      mov   dword ptr [@Mint@_uuMtemp],eax

      mov   eax,dword ptr [@Mint@_uuMtemp.nDigit]
      xchg  eax,dword ptr [@Mint@_vvMtemp.nDigit]
      mov   dword ptr [@Mint@_uuMtemp.nDigit],eax

      mov   eax,dword ptr [@Mint@_uuMtemp.fDigit]
      xchg  eax,dword ptr [@Mint@_vvMtemp.fDigit]
      mov   dword ptr [@Mint@_uuMtemp.fDigit],eax

      mov   eax,dword ptr [@Mint@_uuMtemp.(class Mint)nDim]
      xchg  eax,dword ptr [@Mint@_vvMtemp.(class Mint)nDim]
      mov   dword ptr [@Mint@_uuMtemp.(class Mint)nDim],eax

    }
    Mint::_uuMtemp.isRem(Mint::_uuMtemp,Mint::_vvMtemp,PositiveRem) ;
  }

}


#else

long Kronecker(const Mint& a,const Mint& b)
{
  long k ;
  Mint::_uuMtemp=a ;
  Mint::_vvMtemp=b ;


  if( Mint::_vvMtemp.isZero() ) {                     // b is zero ?
    if( ((unsigned long)Mint::_uuMtemp)==1 && Mint::_uuMtemp.nDigit==1 )
      return 1 ;
    else return 0 ;
  }
Remove_2_from_b:
  if( Mint::_uuMtemp.isEven() && Mint::_vvMtemp.isEven() ) return 0 ;
  int v=0 ;
  while( (((unsigned long)Mint::_vvMtemp)&1)==0 ){  // (a/2)*...*(a/2)*(a/b')
    v++ ;                         // b=2*...*2*b'
    Mint::_vvMtemp>>=1 ;
  }
  if( (v&1)==0 ) k=1 ;
  else k=_Kr_tab[long(Mint::_uuMtemp)&7] ;

  if( Mint::_vvMtemp.isNegative() ) {                 // (a/-1) = 1 if a>=0
    Mint::_vvMtemp.nDim&=_POSITIVE_ ;
    if( Mint::_uuMtemp.isNegative() ) k=-k ;
  }

  for(;;) {
    if( Mint::_uuMtemp.isZero() ) {
      if( *Mint::_vvMtemp.digit==1 && Mint::_vvMtemp.nDigit==1) return k ;
      else return 0 ;
    }
    v=0 ;
    while( (((unsigned long)Mint::_uuMtemp)&1)==0 ) {
         //   (2/p)=(-1)^((p*p-1)/8)
     v++ ;
     Mint::_uuMtemp>>=1 ;
    }
    if( v&1 ) k*=_Kr_tab[((unsigned long)Mint::_vvMtemp)&7] ;

     //(p/q)(q/p)=(-1)^((p-1)(q-1)/4)
    if( long(Mint::_uuMtemp) &((unsigned long)Mint::_vvMtemp) & 2 )
       k=-k ;
    if( Mint::_uuMtemp.isNegative() ) Mint::_uuMtemp.nDim&=_POSITIVE_ ;

    Swap(Mint::_uuMtemp,Mint::_vvMtemp) ;
    Mint::_uuMtemp.isRem(Mint::_uuMtemp,Mint::_vvMtemp) ;
  }


}
#endif
