//
//	Copyright (C) by S.Tomabechi 1995,1998
//	All rights reserved.
//
//	Algorithm 1.6.1 (Roots Mod p)
//	Given a prime number p>=3 and a polynomial P in Fp[X],
//	this algorithm outputs the roots of P in Fp
//

#include <vector>
#include "PolyFp.hpp"

#ifdef USE_MINT
#include "Mint.hpp"
#endif

#include "IntFuncs.hpp"
#include "Error.hpp"

using namespace std ;

PolyFp power(const PolyFp& f,unsigned long n,const PolyFp& p) ;
PolyFp power(const PolyFp& f,unsigned long n) ;

void FindRootModp(PolyFp& a,int& nroots,
					vector< Fp >& roots) ;

int RootModp(PolyFp& f,vector< Fp >& roots)
{

//cout << endl << "Poly" << f << flush ;
	int nroots=0 ;
	PolyFp g,a ;

Isolate_roots_in_Fp:
//cout << endl << "x^p" << power(PolyFp::X,(unsigned long)Char()) ;
	g=power(PolyFp::X,Char(),f) ;	 // g(x)=x^p mod f(x)
//cout << endl << "x^p mod f "<< f << "=" << g <<flush ;
	a=gcd(g-PolyFp::X,f) ;
//cout << endl << "gcd" << a << flush ;
	if( a[0]==0 ) {								// constant term is 0 ?
		roots.push_back(0) ; nroots++ ;
		a>>=1 ;											//		a <= a/x
	}

	FindRootModp(a,nroots,roots) ;
//cout << endl << "nroots " << nroots << flush ;
	return nroots ;
}

void FindRootModp(PolyFp& a,int& nroots,
					vector< Fp >& roots)
{
//cout << endl << "Input polynomial " << a << flush ;
//	a/=a[deg(a)] ;
//cout << endl << "Input polynomial " << a << flush ;
	static unsigned long irand=5 ;
	static PolyFp y(PolyFp::X) ;
	PolyFp b ;
	Fp d,e ;

Small_degree:
	if( deg(a)==0 ) return ;
	else if( deg(a)==1 ) {								// linear equation
	 roots.push_back(-a[0]/a[1]) ; nroots++ ;
//#ifdef NFS_DEBUG
#if 0 
if( a(roots[nroots-1])!=0 ) {
	cout << endl << "linear " ;
	cout << endl << a ;
	cout << endl << roots[nroots-1] << " " << a(roots[nroots-1]) <<flush ;
	Error(rootmodp error) ;
}
#endif
	 return ;
	}
	else if( deg(a)==2 ) {	// quadratic equation
		d=a[1]*a[1]-4*a[0]*a[2] ;
		if( Kronecker((unsigned long)d,Char())==-1 ) return ;	// no square root
		else {
			e=sqrt((unsigned long)d,Char()) ;													 // calculate square roots
			roots.push_back((-a[1]+e)/(2*a[2])) ; nroots++ ;
			roots.push_back((-a[1]-e)/(2*a[2])) ; nroots++ ;
			return ;
		}

	}

Random_splitting:
	do {																					// gcd((X+c)^((p-1)/2),a)
//		y[0]=irandom(irand) ;
		y=PolyFp::X+irandom(irand) ;
		b=gcd(power(y,(Char()-1)/2,a)-1,a) ;
//cout << endl << "b " << b << flush ;
	} while( deg(b)==0 || deg(b)==deg(a) ) ;

Recurse:
	FindRootModp(b,nroots,roots) ;
	FindRootModp(a/b,nroots,roots) ;
}


